@@ -184,6 +184,7 @@ ftp_connection_free (FtpConnection *conn)
 }
 
 #define ftp_connection_in_error(conn) ((conn)->error != NULL)
+#define ftp_connection_clear_error(conn) (g_clear_error (&(conn)->error))
 
 static gboolean
 ftp_connection_pop_job (FtpConnection *conn)
@@ -779,7 +780,7 @@ ftp_connection_prepare (FtpConnection *conn)
   if (ftp_connection_send (conn, 0, "FEAT") != 0) {
     ftp_connection_parse_features (conn);
   } else {
-    g_clear_error (&conn->error);
+    ftp_connection_clear_error(conn);
     conn->workarounds |= FTP_WORKAROUND_FEAT_AFTER_LOGIN;
     conn->features = 0;
   }
@@ -811,21 +812,21 @@ ftp_connection_use (FtpConnection *conn)
    */
   if (conn->features & FTP_FEATURE_EPSV)
     ftp_connection_send (conn, 0, "EPSV ALL");
-  g_clear_error (&conn->error);
+  ftp_connection_clear_error(conn);
 #endif
 
   if (conn->workarounds & FTP_WORKAROUND_FEAT_AFTER_LOGIN) {
     if (ftp_connection_send (conn, 0, "FEAT") != 0) {
       ftp_connection_parse_features (conn);
     } else {
-      g_clear_error (&conn->error);
+      ftp_connection_clear_error(conn);
       conn->features = FTP_FEATURE_EPSV;
     }
   }
 
   if (ftp_connection_send (conn, 0, "SYST"))
     ftp_connection_parse_system (conn);
-  g_clear_error (&conn->error);
+  ftp_connection_clear_error(conn);
 
   return TRUE;
 }
@@ -893,7 +894,7 @@ ftp_connection_ensure_data_connection_epsv (FtpConnection *conn)
     {
       DEBUG ("Successful EPSV response code, but data connection failed. Enabling FTP_WORKAROUND_BROKEN_EPSV.\n");
       conn->workarounds |= FTP_WORKAROUND_BROKEN_EPSV;
-      g_clear_error (&conn->error);
+      ftp_connection_clear_error (conn);
     }
   return connected;
 }
@@ -943,7 +944,7 @@ ftp_connection_ensure_data_connection_pasv (FtpConnection *conn)
       /* set workaround flag (see below), so we don't try this again */
       DEBUG ("Successfull PASV response but data connection failed. Enabling FTP_WORKAROUND_PASV_ADDR.\n");
       conn->workarounds |= FTP_WORKAROUND_PASV_ADDR;
-      g_clear_error (&conn->error);
+      ftp_connection_clear_error (conn);
     }
 
   /* Workaround code:
@@ -1059,7 +1060,7 @@ ftp_connection_try_cd (FtpConnection *conn, const FtpFile *file)
 
   if (!ftp_connection_cd (conn, file))
     {
-      g_clear_error (&conn->error);
+      ftp_connection_clear_error (conn);
       return FALSE;
     }
   
@@ -1302,40 +1303,48 @@ g_vfs_backend_ftp_pop_connection (GVfsBackendFtp *ftp,
 	break;
       conn = g_queue_pop_head (ftp->queue);
 
-      if (conn != NULL) {
-	/* Figure out if this connection had a timeout sent. If so, skip it. */
-	g_mutex_unlock (ftp->mutex);
-	ftp_connection_push_job (conn, job);
-	if (ftp_connection_send (conn, 0, "NOOP"))
-	  break;
-	    
-	g_clear_error (&conn->error);
-	conn->job = NULL;
-	ftp_connection_free (conn);
-	conn = NULL;
-	g_mutex_lock (ftp->mutex);
-	ftp->connections--;
-	continue;
-      }
+      if (conn != NULL)
+	{
+	  /* Figure out if this connection had a timeout sent. If so, skip it. */
+	  g_mutex_unlock (ftp->mutex);
+	  ftp_connection_push_job (conn, job);
+	  if (ftp_connection_send (conn, 0, "NOOP"))
+	    break;
+	      
+	  ftp_connection_clear_error (conn);
+	  conn->job = NULL;
+	  ftp_connection_free (conn);
+	  conn = NULL;
+	  g_mutex_lock (ftp->mutex);
+	  ftp->connections--;
+	  continue;
+	}
 
       if (ftp->connections < ftp->max_connections)
 	{
+	  /* Save current number of connections here, so we can limit maximum 
+	   * connections later.
+	   * This is necessary for threading reasons (connections can be 
+	   * opened or closed while we are still in the opening process. */
+	  guint maybe_max_connections = ftp->connections;
+
 	  ftp->connections++;
 	  g_mutex_unlock (ftp->mutex);
 	  conn = ftp_connection_create (ftp->addr, job);
 	  ftp_connection_prepare (conn);
 	  ftp_connection_login (conn, ftp->user, ftp->password);
 	  ftp_connection_use (conn);
-	  if (!ftp_connection_in_error (conn))
+	  if (G_LIKELY (!ftp_connection_in_error (conn)))
 	    break;
 
-	  ftp_connection_pop_job (conn);
+	  ftp_connection_clear_error (conn);
+	  /* Don't call ftp_connection_pop_job () here, the job isn't done yet */
+	  conn->job = NULL;
 	  ftp_connection_free (conn);
 	  conn = NULL;
 	  g_mutex_lock (ftp->mutex);
 	  ftp->connections--;
-	  /* FIXME: This assignment is racy due to the mutex unlock above */
-	  ftp->max_connections = ftp->connections;
+	  ftp->max_connections = MIN (ftp->max_connections, maybe_max_connections);
 	  if (ftp->max_connections == 0)
 	    {
 	      DEBUG ("no more connections left, exiting...");
@@ -1550,7 +1559,7 @@ try_login:
           !g_error_matches (conn->error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED))
 	break;
 
-      g_clear_error (&conn->error);
+      ftp_connection_clear_error (conn);
     }
   
   ftp_connection_use (conn);
@@ -2176,7 +2185,7 @@ resolve_symlink (GVfsBackendFtp *ftp, FtpConnection *conn, GFileInfo *original,
   if (ftp_connection_in_error (conn))
     {
       g_assert (info == NULL);
-      g_clear_error (&conn->error);
+      ftp_connection_clear_error (conn);
       return original;
     }
   if (info == NULL)
@@ -2326,7 +2335,7 @@ do_enumerate (GVfsBackend *backend,
       
       g_vfs_job_enumerate_done (job);
       conn->job = NULL;
-      g_clear_error (&conn->error);
+      ftp_connection_clear_error (conn);
     }
   else
     g_assert (files == NULL);
